//
// nono
// Copyright (C) 2023 nono project
// Licensed under nono-license.txt
//

// キャッシュ

#pragma once

#include "m68030.h"
#include <array>

// キャッシュライン
struct m68030CacheLine
{
	// タグは命令キャッシュ/データキャッシュで共通だが、
	// 命令キャッシュなら下位1ビットが S (FC2)、
	// データキャッシュなら下位3ビットが FC1,FC0,FC2 の順。
	//    3                   2                   1
	//  1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
	// +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	// |                 Tag Address (24bit)           |             |S| (Inst)
	// +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	//
	// +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	// |                 Tag Address (24bit)           |         |1|0|S| (Data)
	// +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	uint32 tag {};

	std::array<uint32, 4> data {};
	union {
		uint32 valid_all {};
		uint8 valid[4];
	};

	bool IsSuper() const	{ return (tag & 1); }
	uint32 TagAddr() const	{ return (tag & 0xffffff00); }
};

// キャッシュ (命令/データ共通)
class m68030Cache
{
 public:
	// バースト転送可能かつ許可なら true。
	bool burst_enable {};

	// キャッシュ。
	std::array<m68030CacheLine, 16> line {};
};
