//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// エリアセット
//

// 書き込み専用デバイスなので BusIO を使うのも余計手間な気がする。

#include "areaset.h"
#include "mpu.h"
#include "x68kio.h"

// コンストラクタ
AreaSetDevice::AreaSetDevice()
	: inherited(OBJ_AREASET)
{
}

// デストラクタ
AreaSetDevice::~AreaSetDevice()
{
}

// 初期化
bool
AreaSetDevice::Init()
{
	if (inherited::Init() == false) {
		return false;
	}

	x68kio = GetX68kIODevice();

	return true;
}

// リセット
void
AreaSetDevice::ResetHard(bool poweron)
{
	// リセットでどうなるかは分からないけど、
	// さすがに不定ではないだろう。
	x68kio->SetAreaset(0);
}

busdata
AreaSetDevice::Read8(uint32 addr)
{
	// 書き込み専用なので読み出しはすべてバスエラー。
	putlog(2, "Read $%06x.B (BusErr)", addr);
	return busdata::BusErr;
}

busdata
AreaSetDevice::Read16(uint32 addr)
{
	// 書き込み専用なので読み出しはすべてバスエラー。
	putlog(2, "Read $%06x.W (BusErr)", addr);
	return busdata::BusErr;
}

busdata
AreaSetDevice::Write8(uint32 addr, uint32 data)
{
	busdata r;

	// 2バイトおきのようだ。
	uint32 offset = addr & 1;
	if (offset == 0) {
		putlog(2, "Write $%06x.B (BusErr)", addr);
		r.SetBusErr();
	} else {
		putlog(1, "Write $%06x <- $%02x", addr, data);
		x68kio->SetAreaset(data & 0xff);
	}

	const busdata wait = busdata::Wait(7 * 40_nsec);
	r |= wait;
	return r;
}

busdata
AreaSetDevice::Write16(uint32 addr, uint32 data)
{
	// ワード書き込みは可能
	return Write8(addr + 1, data & 0xff);
}

busdata
AreaSetDevice::Peek8(uint32 addr)
{
	return busdata::BusErr;
}
