//
// nono
// Copyright (C) 2023 nono project
// Licensed under nono-license.txt
//

//
// Nereid バンクメモリ
//

#pragma once

#include "device.h"
#include <array>

class BankRAMDevice : public IODevice
{
	using inherited = IODevice;
 public:
	BankRAMDevice(int objid_, int ram_size_MB);
	~BankRAMDevice() override;

	bool Init() override;
	void ResetHard(bool poweron) override;

	busdata Read8(uint32 addr) override;
	busdata Read16(uint32 addr) override;
	busdata Read32(uint32 addr) override;
	busdata Write8(uint32 addr, uint32 data) override;
	busdata Write16(uint32 addr, uint32 data) override;
	busdata Write32(uint32 addr, uint32 data) override;
	busdata Peek8(uint32 addr) override;
	bool Poke8(uint32 addr, uint32 data) override;

	// バンクページ取得・設定
	uint32 GetPage() const { return page; }
	void SetPage(uint32 page);

	// バンクメモリ容量 [MB]を取得 (負数は扱わない)
	uint GetSize() const { return ram_size / 1024 / 1024; }

	// アクセス状況の取得とクリア
	void FetchAccStat(uint8 *buf);

 private:
	// アドレスデコーダ
	inline uint32 Decoder(uint32 addr) const;

	std::unique_ptr<uint8[]> ram {};
	uint ram_size {};

	// ページ番号 (レジスタ値、マスク前)
	uint32 page {};

	// ページ番号をリニアアドレスにした状態で保持。
	uint32 pageaddr {};

	// アクセス状況
	std::array<uint8, 16> accstat_read {};
	std::array<uint8, 16> accstat_write {};
};

// GetBankRAMDevice() は用意していない。
// BankRAMDevice は存在しない場合があるので FindObject() を使うこと。
