//
// nono
// Copyright (C) 2024 nono project
// Licensed under nono-license.txt
//

//
// Goldfish 割り込みコントローラ
//

#pragma once

#include "interrupt.h"

class GFPICDevice : public InterruptDevice
{
	using inherited = InterruptDevice;
	friend class Virt68kInterrupt;

	static const uint32 STATUS		= 0x00;
	static const uint32 PENDING		= 0x04;	// was NUMBER
	static const uint32 CLEAR_ALL	= 0x08;	// was DISABLE_ALL
	static const uint32 DISABLE		= 0x0c;
	static const uint32 ENABLE		= 0x10;

 public:
	GFPICDevice(int id_);
	~GFPICDevice() override;

	bool Init() override;
	void ResetHard(bool poweron) override;

	busdata Read32(uint32 addr) override;
	busdata Write32(uint32 addr, uint32 data) override;
	busdata Peek8(uint32 addr) override;

	busdata InterruptAcknowledge(int lv) override;

	// IRQ の登録。子デバイスが呼ぶ。irq は 1-32。
	void RegistIRQ(int irq, const char *name, Device *source);

	// Virt68kInterrupt の下請け
	void MonitorUpdateSummary(TextScreen& screen, int x, int y);
	int MonitorUpdateDetail(TextScreen& screen, int y, int lv);

 protected:
	void ChangeInterrupt() override;

	InterruptDevice *interrupt {};
};

static inline GFPICDevice *GetGFPICDevice(int id_) {
	return Object::GetObject<GFPICDevice>(OBJ_GFPIC(id_));
}
