//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// GVRAM
//

// GVRAM はワード単位でホストバイトオーダ配置なので、
// バイトアクセス時は HB() マクロを使用のこと。

#include "gvram.h"
#include "mpu.h"
#include "videoctlr.h"

// InsideOut p.135
static const busdata wait = busdata::Wait(9 * 40_nsec);

// コンストラクタ
GVRAMDevice::GVRAMDevice()
	: inherited(OBJ_GVRAM)
{
	uint devlen = 0x20'0000;	// XXX とりあえず

	mem.reset(new uint8[devlen]);
}

// デストラクタ
GVRAMDevice::~GVRAMDevice()
{
}

busdata
GVRAMDevice::Read8(uint32 addr)
{
	busdata data;
	uint32 offset = addr - baseaddr;
	data = mem[HB(offset)];
	data |= wait;
	return data;
}

busdata
GVRAMDevice::Read16(uint32 addr)
{
	busdata data;
	uint32 offset = addr - baseaddr;
	data = *(uint16 *)&mem[offset];
	data |= wait;
	return data;
}

busdata
GVRAMDevice::Write8(uint32 addr, uint32 data)
{
	uint32 offset = addr - baseaddr;
	mem[HB(offset)] = data;
	return wait;
}

busdata
GVRAMDevice::Write16(uint32 addr, uint32 data)
{
	uint32 offset = addr - baseaddr;
	*(uint16 *)&mem[offset] = data;
	return wait;
}

busdata
GVRAMDevice::Peek8(uint32 addr)
{
	return mem[HB(addr - baseaddr)];
}
