//
// nono
// Copyright (C) 2024 nono project
// Licensed under nono-license.txt
//

//
// メインバス (virt68k)
//

#pragma once

#include "mainbus.h"

class Virt68kMainbus : public MainbusDevice
{
	using inherited = MainbusDevice;
 public:
	Virt68kMainbus();
	~Virt68kMainbus() override;

	bool InitMainbus() override;

	busdata Peek8(uint32 addr) override;
	bool Poke8(uint32 addr, uint32 data) override;

	busdata Read8(busaddr addr) override;
	busdata Read16(busaddr addr) override;
	busdata Read32(busaddr addr) override;
	busdata Write8(busaddr addr, uint32 data) override;
	busdata Write16(busaddr addr, uint32 data) override;
	busdata Write32(busaddr addr, uint32 data) override;
	busdata ReadBurst16(busaddr addr, uint32 *dst) override;
	busdata WriteBurst16(busaddr addr, const uint32 *dst) override;

 private:
	inline IODevice *Decoder(uint32) const;

	// ブートページを切り替える。
	void SwitchBootPage(bool isrom) override;

	std::unique_ptr<IODevice> pBusErr {};
	std::unique_ptr<IODevice> pInterrupt {};
	std::unique_ptr<IODevice> pMainRAM {};
	std::unique_ptr<IODevice> pV68kIO {};

	uint32 ramsize {};
};
