//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// プリンタポート
//

#include "printer.h"
#include "mpu.h"

// InsideOut p.135
static const busdata wait = busdata::Wait(7 * 40_nsec);

// コンストラクタ
PrinterDevice::PrinterDevice()
	: inherited(OBJ_PRINTER)
{
}

// デストラクタ
PrinterDevice::~PrinterDevice()
{
}

busdata
PrinterDevice::Read8(uint32 addr)
{
	// 書き込み専用
	busdata data = busdata::BusErr;
	data |= wait;
	return data;
}

busdata
PrinterDevice::Read16(uint32 addr)
{
	// 書き込み専用
	busdata data = busdata::BusErr;
	data |= wait;
	return data;
}

busdata
PrinterDevice::Write8(uint32 addr, uint32 data)
{
	uint32 offset = addr - baseaddr;
	busdata r;

	switch (offset) {
	 case 0:
		// データ書き込み
		putlog(0, "Write Data (NOT IMPLEMENTED)");
		r.SetBusErr();
		break;
	 case 3:
		// ストローブ書き込み
		putlog(0, "Write Strobe (NOT IMPLEMENTED)");
		r = 0;
		break;
	 default:
		break;
	}

	r |= wait;
	return r;
}

busdata
PrinterDevice::Write16(uint32 addr, uint32 data)
{
	return Write8(addr + 1, data);
}

busdata
PrinterDevice::Peek8(uint32 addr)
{
	// XXX
	return 0xff;
}
