//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// ROM の基本クラス
//

#pragma once

#include "device.h"

class ROMDevice : public IODevice
{
	using inherited = IODevice;
 public:
	ROMDevice(int objid_);
	~ROMDevice() override;

	busdata Read8(uint32 addr) override;
	busdata Read16(uint32 addr) override;
	busdata Read32(uint32 addr) override;
	busdata Write8(uint32 addr, uint32 data) override;
	busdata Write16(uint32 addr, uint32 data) override;
	busdata Write32(uint32 addr, uint32 data) override;
	busdata Peek8(uint32 addr) override;

	// ファイル名を取得。なければ NULL。
	const char *GetFilename() const;

	// アクセスウェイト [clock] を取得/設定
	void SetWait(uint32 wait_);

 protected:
	bool LoadROM(const char *name, int size);

	// Read*()/Peek8() がアクセスするメモリ領域をさすポインタ。
	const uint8 *mem {};

	// アクセスマスク。
	// LUNA の PROM のように、この ROM の折り返しミラーが見える構造なら
	// このアクセスマスクを設定すること。
	// X680x0 の各 ROM などはミラーが不要なのでコンストラクタで初期化される
	// 0xffffffff のままでよい。
	uint32 mask {};

	// ファイル名
	std::string filename {};

	// イメージ
	// (ファイルから読み込んだ内容、あるいは自分で作ったバッファ)
	std::unique_ptr<uint8[]> imagebuf {};

	// ライトアクセス時の動作 (Write*() の戻り値)。
	// 派生クラスが指定すること。
	busdata write_op {};

	// アクセスウェイト
	busdata wait {};
};
