//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// サブメモリ
//

#pragma once

#include "device.h"

// サブメモリ (基本クラス)
class SubRAMBaseDevice : public IODevice
{
	using inherited = IODevice;
 public:
	SubRAMBaseDevice(int objid_, uint ramsize_kb);
	~SubRAMBaseDevice() override;

	void ResetHard(bool poweron) override;

	busdata Read8(uint32 addr) override;
	busdata Read16(uint32 addr) override;
	busdata Read32(uint32 addr) override;
	busdata Write8(uint32 addr, uint32 data) override;
	busdata Write16(uint32 addr, uint32 data) override;
	busdata Write32(uint32 addr, uint32 data) override;
	busdata Peek8(uint32 addr) override;
	bool Poke8(uint32 addr, uint32 data) override;

 private:
	// アドレスデコーダ
	uint32 Decoder(uint32 addr) const;

	std::unique_ptr<uint8[]> ram {};
	uint ramsize {};
};

// サブメモリ (XP/Lance との共有 RAM)
class SubRAMDevice : public SubRAMBaseDevice
{
	using inherited = SubRAMBaseDevice;
 public:
	SubRAMDevice(int ramsize_kb);
	~SubRAMDevice() override;

	void ResetHard(bool poweron) override;
};

// サブメモリ2 (XP の非共有部)
class SubRAM2Device : public SubRAMBaseDevice
{
	using inherited = SubRAMBaseDevice;
 public:
	SubRAM2Device(int ramsize_kb);
	~SubRAM2Device() override;
};

static inline SubRAMDevice *GetSubRAMDevice() {
	return Object::GetObject<SubRAMDevice>(OBJ_SUBRAM);
}
