//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// TVRAM
//

#pragma once

#include "planevram.h"

class Renderer;

class TVRAMDevice : public PlaneVRAMDevice
{
	using inherited = PlaneVRAMDevice;

	static const int baseaddr = 0xe00000;

	// プレーン数は4枚固定
	static const int NPLANE = 4;

	struct tvram {
		bool sa;				// 同時アクセス
		bool ap[NPLANE];		// 同時アクセスプレーン
		bool cp[NPLANE];		// ラスターコピー対象
		bool men;				// アクセスマスクイネーブル
		uint16 mask;			// アクセスマスク
	};

 public:
	TVRAMDevice();
	~TVRAMDevice() override;

	bool Init() override;

	busdata Read8(uint32 addr) override;
	busdata Read16(uint32 addr) override;
	busdata Write8(uint32 addr, uint32 data) override;
	busdata Write16(uint32 addr, uint32 data) override;
	busdata Peek8(uint32 addr) override;
	bool Poke8(uint32 addr, uint32 data) override;

	void UpdateInfo(TextScreen&, int x, int y) const override;

	void SetAccessPlane(uint16 data);
	void SetAccessMask(uint16 data);
	void SetScrollX(int x);
	void SetScrollY(int y);
	void RasterCopy(int src_ras, int dst_ras);

 private:
	// アドレスデコーダ
	uint32 Decoder(uint32 addr) const;

	inline void SetDirty(uint32 offset);

	// X 方向にはみ出したときに対応する仮想画面の Y 座標を返す。
	int GetWrapY(int y) const override;

	struct tvram tvram {};

	Renderer *renderer {};
};

static inline TVRAMDevice *GetTVRAMDevice() {
	return Object::GetObject<TVRAMDevice>(OBJ_PLANEVRAM);
}
