//
// nono
// Copyright (C) 2022 nono project
// Licensed under nono-license.txt
//

//
// VM (NEWS)
//

#include "vm_news.h"
#include "config.h"
#include "dipsw.h"
#include "mainapp.h"
#include "mainbus_news.h"
#include "mpu680x0.h"
#include "newsfb.h"
#include "power.h"
#include "renderer.h"

// コンストラクタ
VM_NEWS::VM_NEWS()
	: inherited("NWS-1750")
{
	// 機種固有パラメータ
	// MPU クロックの初期値は機種ごとに異なる。単位は MHz
	gConfig->SetDefault("mpu-clock", 25);
	// RAM 初期値は?
	gConfig->SetDefault("ram-size", 16);
	// RTC epoch は 1900年。
	gConfig->SetDefault("rtc-epoch-year", 1900);

	NEWDV(Power, new PowerDevice());
	NEWDV(MPU, new MPU680x0Device());
	NEWDV(Mainbus, new NewsMainbus());
	NEWDV(Renderer, new NopRenderer());
	NEWDV(Newsfb, new NewsfbDevice());
	NEWDV(Dipsw, new NewsDipswDevice());

	// バーストアクセスがあるかどうかは分からないが
	// Mainbus が LUNA-I と共通なので一応揃えておく。
	dynamic_cast<MPU680x0Device*>(pMPU.get())->EnableBurstAccess(true);
}

// デストラクタ
VM_NEWS::~VM_NEWS()
{
}
